#pragma once

#include <Arduino.h>
#include <ArduinoJson.h>
#include <Preferences.h>
#include <WiFi.h>
#include <esp_task_wdt.h>
#include <WiFiClientSecure.h>

#include <nvs_flash.h>
#include <WiFiManager.h>
#include <ESPmDNS.h>

#include "block_notify.hpp"
#include "led_handler.hpp"
#include "nostr_notify.hpp"
#include "price_notify/price_notify.hpp"
#include "screen_handler.hpp"
#include "shared.hpp"
#include "timers.hpp"
#include "v2_notify.hpp"
#include "webserver.hpp"
#include "button_handler.hpp"
#include "bitaxe_fetch.hpp"
#include "mining_pool_stats_fetch.hpp"
#include "epd.hpp"

#ifdef HAS_FRONTLIGHT
#include <BH1750.h>
#include <PCA9685.h>
#endif

extern Preferences preferences;
extern QueueHandle_t workQueue;
extern PriceNotify::PriceNotifyManager priceManager;

void setupConfig();
void setupWifi();
void setupOTA();
void setupMDNS();
void setupDataSources();
void stopDataSources();
void restartDataSources();
void setupTasks();
void setupTimers();
void setupLittleFS();
void setupPreferences();
void setupWDT();
void setupWorkQueue();
void setupLedHandler();
void setupScreenHandler();
void setupWebserver();
void setupNostrNotify();
void setupBlockNotify();
void setupV2Notify();
void setupPriceNotify();
void setupHardware();
void finishSetup();
void syncTime();

void handleOTA();
void handleWifi();
void handleWDT();
void handleWorkQueue();

void setWifiTxPower(int8_t power);
void onWifiEvent(WiFiEvent_t event);
void WiFiEvent(WiFiEvent_t event, WiFiEventInfo_t info);

DataSourceType getDataSource();
void setDataSource(DataSourceType source);
String getMyHostname();
uint getLastTimeSync();
bool debugLogEnabled();

#ifdef HAS_FRONTLIGHT
float getLightLevel();
bool hasLightLevel();

extern PCA9685 flArray;
#endif

String getHwRev();
bool isWhiteVersion();
String getFsRev();

#define NTP_SERVER "pool.ntp.org"
#define DEFAULT_TIME_OFFSET_SECONDS 3600