#pragma once

#include <Arduino.h>
#include <WebSocketsClient.h>
#include <functional>
#include <map>
#include <string>

namespace PriceNotify {

using std::function;
using std::map;
using std::string;

// Convert char-based currency to enum for type safety
enum class Currency : char {
    USD = '$',
    EUR = '[',
    GBP = ']',
    JPY = '^',
    AUD = '_',
    CAD = '`'
};

class IPriceSource {
public:
    virtual ~IPriceSource() = default;
    
    // Initialize and connect to the websocket
    virtual void connect() = 0;
    
    // Disconnect and cleanup
    virtual void disconnect() = 0;
    
    // Check connection status
    virtual bool isConnected() const = 0;
    
    // Get the last known price for a currency
    virtual uint64_t getPrice(Currency currency) const = 0;
    
    // Get the last update timestamp for a currency
    virtual uint32_t getLastUpdate(Currency currency) const = 0;
    
    // Set callback for price updates
    virtual void onPriceUpdate(function<void(Currency, uint64_t)> callback) = 0;
    
    // Process websocket loop - should be called regularly
    virtual void loop() = 0;

protected:
    function<void(Currency, uint64_t)> priceUpdateCallback;
};
} // namespace PriceNotify 