#pragma once

#include "../interfaces/price_source.hpp"
#include <ArduinoJson.h>
#include <esp_timer.h>

namespace PriceNotify {

using std::map;

class KrakenSource : public IPriceSource {
public:
    KrakenSource();
    ~KrakenSource() override;

    void connect() override;
    void disconnect() override;
    bool isConnected() const override;
    uint64_t getPrice(Currency currency) const override;
    uint32_t getLastUpdate(Currency currency) const override;
    void onPriceUpdate(function<void(Currency, uint64_t)> callback) override;
    void loop() override;

private:
    static void onWebSocketEvent(WStype_t type, uint8_t* payload, size_t length);
    void handleWebSocketEvent(WStype_t type, uint8_t* payload, size_t length);
    void processMessage(const char* payload);
    void subscribe();

    WebSocketsClient webSocket;
    map<Currency, uint64_t> prices;
    map<Currency, uint32_t> lastUpdates;
    bool connected;

    static KrakenSource* instance;  // For callback handling
};
} // namespace PriceNotify 